@@ -166,3 +166,23 @@ func (m *Manager) saveBuiltinMonitorConfig(name string) (err error) {
 	m.sysConfig.mu.Unlock()
 	return
 }
+
+//仅保存主屏的配置，greeter登录后读取该配置进行初始化分辨率设置
+func (m *Manager) saveConnectMonitorConfig(monitor *Monitor) (err error) {
+	if monitor == nil {
+		return
+	}
+
+	m.sysConfig.mu.Lock()
+	defer m.sysConfig.mu.Unlock()
+
+	m.sysConfig.Config.Cache.GreeterMonitor.Name = monitor.Name
+	m.sysConfig.Config.Cache.GreeterMonitor.Model = monitor.Model
+	m.sysConfig.Config.Cache.GreeterMonitor.BestMode = monitor.BestMode
+	m.sysConfig.Config.Cache.GreeterMonitor.Enabled = monitor.Enabled
+	m.sysConfig.Config.Cache.GreeterMonitor.Width = monitor.Width
+	m.sysConfig.Config.Cache.GreeterMonitor.Height = monitor.Height
+
+	err = m.saveSysConfigNoLock("greeter monitor")
+	return
+}
\ No newline at end of file
