@@ -12,6 +12,7 @@ import (
 	sysdisplay "github.com/linuxdeepin/go-dbus-factory/com.deepin.system.display"
 	"github.com/linuxdeepin/go-lib/dbusutil"
 	"github.com/linuxdeepin/go-lib/log"
+	"github.com/linuxdeepin/startdde/xsettings"
 )
 
 var logger = log.NewLogger("daemon/display")
@@ -74,6 +75,15 @@ func (h *scaleFactorsHelper) SetChangedCb(fn func(factors map[string]float64) er
 	h.changedCb = fn
 }
 
+func (m *Manager) updateScaleFactors(factors map[string]float64) error {
+	// 已初始化就已经写过配置文件
+	if _dpy != nil {
+		return nil
+	}
+
+	return m.setScaleFactors(factors)
+}
+
 func (m *Manager) setScaleFactors(factors map[string]float64) error {
 	logger.Debug("setScaleFactors", factors)
 	m.sysConfig.mu.Lock()
@@ -90,8 +100,8 @@ func (m *Manager) setScaleFactors(factors map[string]float64) error {
 	return err
 }
 
-func Start(service *dbusutil.Service) error {
-	m := newManager(service)
+func Start(xsm *xsettings.XSManager, service *dbusutil.Service) error {
+	m := newManager(xsm, service)
 	m.init()
 
 	if !_greeterMode {
@@ -117,7 +127,6 @@ func StartPart2() error {
 	m := _dpy
 	m.initSysDisplay()
 	m.initTouchscreens()
-
 	if !_greeterMode {
 		controlRedshift("disable")
 		m.applyColorTempConfig(m.DisplayMode)
