@@ -43,6 +43,7 @@ func (m *Manager) handleMonitorChanged(monitorInfo *MonitorInfo) {
 		options[optionDisableCrtc] = true
 	}
 	m.updateMonitorsId(options)
+	m.tryToChangeScaleFactor(monitorInfo.Width, monitorInfo.Height)
 }
 
 // wayland 下连接显示器
@@ -54,6 +55,8 @@ func (m *Manager) handleMonitorAdded(monitorInfo *MonitorInfo) {
 	}
 	m.updatePropMonitors()
 	m.updateMonitorsId(nil)
+	// wayland下所有显示器都可以缩放到1-3倍，固此处修改失去意义。
+	//m.tryToChangeScaleFactor(monitorInfo.Width, monitorInfo.Height)
 }
 
 // wayland 下断开显示器
