@@ -23,19 +23,25 @@ import (
 
 	"github.com/davecgh/go-spew/spew"
 	"github.com/godbus/dbus"
+	"github.com/jouyouyun/hardware/graphic"
 	sessiondisplay "github.com/linuxdeepin/go-dbus-factory/com.deepin.daemon.display"
 	sysdisplay "github.com/linuxdeepin/go-dbus-factory/com.deepin.system.display"
 	inputdevices "github.com/linuxdeepin/go-dbus-factory/com.deepin.system.inputdevices"
+	systeminfo "github.com/linuxdeepin/go-dbus-factory/com.deepin.system.systeminfo"
+	configManager "github.com/linuxdeepin/go-dbus-factory/org.desktopspec.ConfigManager"
 	ofdbus "github.com/linuxdeepin/go-dbus-factory/org.freedesktop.dbus"
 	login1 "github.com/linuxdeepin/go-dbus-factory/org.freedesktop.login1"
 	gio "github.com/linuxdeepin/go-gir/gio-2.0"
 	"github.com/linuxdeepin/go-lib/dbusutil"
 	"github.com/linuxdeepin/go-lib/gsettings"
 	"github.com/linuxdeepin/go-lib/log"
+	"github.com/linuxdeepin/go-lib/strv"
 	"github.com/linuxdeepin/go-lib/xdg/basedir"
 	x "github.com/linuxdeepin/go-x11-client"
 	"github.com/linuxdeepin/go-x11-client/ext/randr"
+
 	"github.com/linuxdeepin/startdde/display/brightness"
+	"github.com/linuxdeepin/startdde/xsettings"
 )
 
 const (
@@ -73,6 +79,7 @@ const (
 	gsKeyRateFilter  = "rate-filter"
 	//gsKeyPrimary     = "primary"
 	gsXSettingsPrimaryName       = "primary-monitor-name"
+	gsXSettingsPrimaryRect       = "primary-monitor-rect"
 	gsKeyCustomMode              = "current-custom-mode"
 	gsKeyColorTemperatureMode    = "color-temperature-mode"
 	gsKeyColorTemperatureManual  = "color-temperature-manual"
@@ -98,6 +105,15 @@ const (
 	priorityOther
 )
 
+const (
+	DSettingsAppID                                    = "org.deepin.startdde"
+	DSettingsDisplayName                              = "org.deepin.Display"
+	DSettingsKeyHDMIIsBuiltinConfig                   = "HDMI-is-builtin"
+	DSettingsKeyBackLightMaxBrightnessChooseBigConfig = "backLight-max-brightness-choose-big"
+	DSettingsKeySpecialGPUVendorListConfig            = "special-gpu-vendor-list"
+	DSettingsKeyLABCAutoStopEnabledConfig             = "labc-auto-stop-enabled"
+)
+
 var (
 	_monitorTypePriority = map[string]int{
 		"edp":  priorityEDP,
@@ -175,6 +191,7 @@ type Manager struct {
 	applySaveMu              sync.Mutex
 	inApply                  bool
 	futureConfig             monitorsFutureConfig
+	xsManager                *xsettings.XSManager
 
 	// dbusutil-gen: equal=objPathsEqual
 	Monitors []dbus.ObjectPath
@@ -217,6 +234,13 @@ type Manager struct {
 	// 不支持调节色温的显卡型号
 	unsupportGammaDrmList []string
 	drmSupportGamma       bool
+
+	dsHDMIIsBuiltinConfig                   []string
+	dsBackLightMaxBrightnessChooseBigConfig []string
+	dmiInfo                                 systeminfo.DMIInfo
+	pciVendor                               []string
+	dsSpecialGPUVendorList                  []string
+	dsLABCAutoStopEnable                    bool
 }
 
 type monitorSizeInfo struct {
@@ -226,7 +250,7 @@ type monitorSizeInfo struct {
 
 var _ monitorManagerHooks = (*Manager)(nil)
 
-func newManager(service *dbusutil.Service) *Manager {
+func newManager(xsm *xsettings.XSManager, service *dbusutil.Service) *Manager {
 	m := &Manager{
 		service:        service,
 		monitorMap:     make(map[uint32]*Monitor),
@@ -235,6 +259,7 @@ func newManager(service *dbusutil.Service) *Manager {
 		unsupportGammaDrmList: []string{
 			"Loongson",
 		},
+		xsManager: xsm,
 	}
 	m.redshiftRunner.cb = func(value int) {
 		m.setColorTempOneShot()
@@ -282,8 +307,6 @@ func newManager(service *dbusutil.Service) *Manager {
 
 	m.mm.setHooks(m)
 
-	m.setPropMaxBacklightBrightness(uint32(brightness.GetMaxBacklightBrightness()))
-
 	sysSigLoop := dbusutil.NewSignalLoop(m.sysBus, 10)
 	m.sysSigLoop = sysSigLoop
 	sysSigLoop.Start()
@@ -362,11 +385,6 @@ func newManager(service *dbusutil.Service) *Manager {
 	if err != nil {
 		logger.Warning(err)
 	}
-
-	m.drmSupportGamma, _ = m.detectDrmSupportGamma()
-	if m.drmSupportGamma {
-		m.setColorTempModeReal(ColorTemperatureModeNone)
-	}
 	return m
 }
 
@@ -399,6 +417,28 @@ func (m *Manager) initSysDisplay() {
 	if err != nil {
 		logger.Warning(err)
 	}
+	_, err = m.sysDisplay.ConnectBacklightBrightnessUpdated(func(brightness float64) {
+		if !m.sessionActive {
+			return
+		}
+		if v := m.getBuiltinMonitor(); v != nil {
+			m.setBrightnessAndSync(v.Name, brightness)
+			err = m.saveBrightnessInCfg(map[string]float64{
+				v.Name: brightness,
+			})
+		}
+	})
+	if err != nil {
+		logger.Warning(err)
+	}
+	go func() {
+		// 依赖dsettings数据，需要在initDSettings后执行
+		m.drmSupportGamma, _ = m.detectDrmSupportGamma()
+		if m.drmSupportGamma {
+			logger.Debug("setColorTempModeReal")
+			m.setColorTempModeReal(ColorTemperatureModeNone)
+		}
+	}()
 }
 
 // 处理系统级别的配置更新
@@ -536,6 +576,32 @@ func (m *Manager) handleSysConfigUpdated(newSysConfig *SysRootConfig) {
 	}
 }
 
+//获取用户设置的分辨率，用于greeter初始化使用
+func (m *Manager) updateConnectMonitors() {
+	//优先使用内建屏，其次再用主屏
+	builtinMonitorName := m.sysConfig.Config.Cache.BuiltinMonitor
+	connectMonitors := m.getConnectedMonitors()
+	var saveMonitorName string
+
+	if builtinMonitorName != "" {
+		saveMonitorName = m.sysConfig.Config.Cache.BuiltinMonitor
+	} else {
+		primaryMonitor := m.mm.getPrimaryMonitor()
+		if primaryMonitor != nil {
+			saveMonitorName = primaryMonitor.Name
+		}
+	}
+
+	logger.Info("[updateConnectMonitors] saveMonitorName:", saveMonitorName)
+	for _, monitor := range connectMonitors {
+		if monitor.Name == saveMonitorName {
+			m.saveConnectMonitorConfig(monitor)
+			logger.Info("[updateConnectMonitors] save greeter use mode. monitor:", monitor)
+			return
+		}
+	}
+}
+
 // initBuiltinMonitor 初始化内置显示器。
 func (m *Manager) initBuiltinMonitor() {
 	if !m.hasBuiltinMonitor {
@@ -950,6 +1016,8 @@ func (m *Manager) init() {
 		m.monitorsId = m.getMonitorsId()
 		m.updatePropMonitors()
 
+		m.updateConnectMonitors()
+
 		pmi := m.mm.getPrimaryMonitor()
 		if pmi != nil {
 			m.setPropPrimary(pmi.Name)
@@ -998,47 +1066,107 @@ func (m *Manager) init() {
 		// 没有内建屏,不监听内核信号
 		logger.Info("built-in screen does not exist")
 	}
-
+	m.initDSettings(m.sysBus)
 	go func() {
-		// 每次设置过主屏后，都将此值同步到xsettings
+		m.initHardwareInfo(m.sysBus)
+		m.setPropMaxBacklightBrightness(uint32(m.GetMaxBacklightBrightness()))
+	}()
+	go func() {
+		// 每次设置过主屏和其rect区域后，都将此值同步到xsettings
 		bus, _ := dbus.SessionBus()
 		display := sessiondisplay.NewDisplay(bus)
 		sigLoop := dbusutil.NewSignalLoop(bus, 10)
 		sigLoop.Start()
 		display.InitSignalExt(sigLoop, true)
+
 		err = display.Primary().ConnectChanged(func(hasValue bool, primary string) {
 			if !hasValue {
 				return
 			}
 
-			// 保持主屏数据和xsettings同步
-			for _, key := range m.xSettingsGs.ListKeys() {
-				if gsXSettingsPrimaryName == key {
-					oldPrimary := m.xSettingsGs.GetString(gsXSettingsPrimaryName)
-					if oldPrimary != primary {
-						m.xSettingsGs.SetString(gsXSettingsPrimaryName, primary)
-					}
-					return
+			// 保持主屏Name和xsettings同步
+			if m.xSettingsGs.GetSchema().HasKey(gsXSettingsPrimaryName) {
+				oldPrimary := m.xSettingsGs.GetString(gsXSettingsPrimaryName)
+				if oldPrimary != primary {
+					m.xSettingsGs.SetString(gsXSettingsPrimaryName, primary)
 				}
+				return
 			}
 		})
 		if err != nil {
 			logger.Warning("connect to `Primary` property changed failed:", err)
 		}
 
-		// 启动后先同步一次
-		for _, key := range m.xSettingsGs.ListKeys() {
-			if gsXSettingsPrimaryName == key {
-				oldPrimary := m.xSettingsGs.GetString(gsXSettingsPrimaryName)
-				if oldPrimary != m.Primary {
-					m.xSettingsGs.SetString(gsXSettingsPrimaryName, m.Primary)
+		err = display.PrimaryRect().ConnectChanged(func(hasValue bool, rect sessiondisplay.Rectangle) {
+			if !hasValue {
+				return
+			}
+
+			// 保持主屏Rect和xsettings同步
+			if m.xSettingsGs.GetSchema().HasKey(gsXSettingsPrimaryRect) {
+				oldRect := m.xSettingsGs.GetString(gsXSettingsPrimaryRect)
+				if oldRect != rect2String(rect) {
+					m.xSettingsGs.SetString(gsXSettingsPrimaryRect, rect2String(rect))
 				}
 				return
 			}
+		})
+		if err != nil {
+			logger.Warning("connect to `PrimaryRect` property changed failed:", err)
+		}
+
+		// 启动后先同步一次
+		if m.xSettingsGs.GetSchema().HasKey(gsXSettingsPrimaryName) {
+			oldPrimary := m.xSettingsGs.GetString(gsXSettingsPrimaryName)
+			if oldPrimary != m.Primary {
+				m.xSettingsGs.SetString(gsXSettingsPrimaryName, m.Primary)
+			}
+			return
+		}
+		if m.xSettingsGs.GetSchema().HasKey(gsXSettingsPrimaryRect) {
+			oldRect := m.xSettingsGs.GetString(gsXSettingsPrimaryRect)
+			if oldRect != xrect2String(m.PrimaryRect) {
+				m.xSettingsGs.SetString(gsXSettingsPrimaryRect, xrect2String(m.PrimaryRect))
+			}
+			return
 		}
 	}()
 }
 
+func xrect2String(rect x.Rectangle) string {
+	return fmt.Sprintf("%d-%d-%d-%d", rect.X, rect.Y, rect.Width, rect.Height)
+}
+
+func rect2String(rect sessiondisplay.Rectangle) string {
+	return fmt.Sprintf("%d-%d-%d-%d", rect.X, rect.Y, rect.Width, rect.Height)
+}
+
+// 从控制中心迁移过来计算屏幕缩放范围。此函数返回屏幕最大的缩放值
+func calcMaxScaleFactor(width, height uint16) float64 {
+	scaleFactors := []float64{1.0, 1.25, 1.5, 1.75, 2.0, 2.25, 2.5, 2.75, 3.0}
+
+	maxWScale := float64(width) / 1024.0
+	maxHScale := float64(height) / 768.0
+
+	maxValue := 0.0
+	if maxWScale < maxHScale {
+		maxValue = maxWScale
+	} else {
+		maxValue = maxHScale
+	}
+
+	if maxValue > 3.0 {
+		maxValue = 3.0
+	}
+
+	maxScale := 0.0
+	for idx := 0; (float64(idx)*0.25 + 1.0) <= maxValue; idx++ {
+		maxScale = scaleFactors[idx]
+	}
+
+	return maxScale
+}
+
 // calcRecommendedScaleFactor 计算推荐的缩放比
 func calcRecommendedScaleFactor(widthPx, heightPx, widthMm, heightMm float64) float64 {
 	if widthMm == 0 || heightMm == 0 {
@@ -1273,6 +1401,24 @@ func (m *Manager) addMonitor(monitorInfo *MonitorInfo) error {
 	return nil
 }
 
+func (m *Manager) tryToChangeScaleFactor(monitorWidth, monitorHeight uint16) {
+	// x 下拔掉显示器会触发更新操作，高宽均为0
+	if monitorWidth == 0 || monitorHeight == 0 {
+		return
+	}
+
+	curScale := xsettings.GetScaleFactor()
+	maxScale := calcMaxScaleFactor(monitorWidth, monitorHeight)
+	if curScale > maxScale && m.xsManager != nil {
+		recommendScaleFactor := GetRecommendedScaleFactor()
+		// 更新scale factor
+		m.xsManager.SetScaleFactor(recommendScaleFactor)
+		m.updateScaleFactors(map[string]float64{
+			"ALL": recommendScaleFactor,
+		})
+	}
+}
+
 // updateMonitor 根据 outputInfo 中的信息更新 dbus 上的 Monitor 对象的属性
 func (m *Manager) updateMonitor(monitorInfo *MonitorInfo) {
 	m.monitorMapMu.Lock()
@@ -1533,8 +1679,16 @@ func (m *Manager) handlePrimaryRectChanged(pmi *MonitorInfo) {
 	m.PropsMu.Lock()
 	defer m.PropsMu.Unlock()
 
-	// 复制模式不用设置主屏
-	if m.DisplayMode != DisplayModeMirror {
+	// 复制模式不用设置主屏，由于单屏和复制模式共用一个配置，因此此处需要判断屏幕个数来确实是否是复制模式
+	if m.DisplayMode == DisplayModeMirror {
+		connectedMonitors := m.getConnectedMonitors()
+		primaryMonitor := connectedMonitors.GetByName(m.Primary)
+		if len(connectedMonitors) > 1 && primaryMonitor != nil {
+			logger.Debug("The current mode is DisplayModeMirror and the primary screen exists.")
+			return
+		}
+	}
+	if pmi != nil {
 		m.setPropPrimary(pmi.Name)
 		m.setPropPrimaryRect(pmi.getRect())
 	}
@@ -1685,7 +1839,6 @@ func (m *Manager) buildConfigForModeOnlyOne(monitors Monitors, uuid string) (mon
 			//cfg.Reflect = 0
 			cfg.Brightness = 1
 			monitorCfgs = append(monitorCfgs, cfg)
-			return
 		}
 	}
 	return
@@ -1743,6 +1896,13 @@ func (m *Manager) applyModeOnlyOne(monitorsId monitorsId, monitorMap map[uint32]
 		}
 	}
 
+	if len(m.Monitors) > 1 && len(configs) > 1 && name != "" {
+		for _, config := range configs {
+			config.Enabled = config.Name == name
+			logger.Debug("update OnlyOneConfig", config)
+		}
+	}
+
 	err = m.applySysMonitorConfigs(DisplayModeOnlyOne, monitorsId, monitorMap, configs, options)
 	if err != nil {
 		return
@@ -1973,6 +2133,8 @@ func (m *Manager) applyChanges() error {
 		config.modify(monitor.changes)
 	}
 
+	m.updateConnectMonitors()
+
 	err := m.applySysMonitorConfigs(DisplayModeInvalid, monitorsId, monitorMap, configs, nil)
 	if err != nil {
 		logger.Warning("[applyChanges] apply sys monitor configs failed:", err)
@@ -2036,6 +2198,7 @@ func (m *Manager) applySysMonitorConfigs(mode byte, monitorsId monitorsId, monit
 		return errors.New("invalid configs: no enabled monitor")
 	}
 
+	var primaryName string
 	var primaryMonitorID uint32
 	var enabledMonitors []*Monitor
 	for _, monitor := range monitorMap {
@@ -2051,6 +2214,7 @@ func (m *Manager) applySysMonitorConfigs(mode byte, monitorsId monitorsId, monit
 				logger.Debug("enable monitor", monitor)
 				if monitorCfg.Primary {
 					primaryMonitorID = monitor.ID
+					primaryName = monitor.Name
 				}
 				enabledMonitors = append(enabledMonitors, monitor)
 				//所有可设置的值都设置为配置文件中的值
@@ -2103,6 +2267,7 @@ func (m *Manager) applySysMonitorConfigs(mode byte, monitorsId monitorsId, monit
 	go func() {
 		for _, config := range configs {
 			if config.Enabled && m.canSetBrightness(config.Name) {
+				logger.Debug("Start set brightness")
 				err := m.setBrightness(config.Name, config.Brightness)
 				if err != nil {
 					logger.Warningf("call setBrightness err: %v, config.Name: %s", err, config.Name)
@@ -2130,6 +2295,10 @@ func (m *Manager) applySysMonitorConfigs(mode byte, monitorsId monitorsId, monit
 		logger.Warning(err)
 	}
 
+	if primaryName != "" && m.Primary != primaryName && mode == DisplayModeExtend {
+		m.setPropPrimary(primaryName)
+	}
+
 	return nil
 }
 
@@ -2867,7 +3036,7 @@ func getLspci(args []string) (string, error) {
 }
 
 func (m *Manager) detectDrmSupportGamma() (bool, error) {
-	args := []string{}
+	var args []string
 	pciInfo, err := getLspci(args)
 	if err != nil {
 		logger.Warning(err)
@@ -2892,15 +3061,141 @@ func (m *Manager) detectDrmSupportGamma() (bool, error) {
 		}
 	}
 	if !vgaSupportGamma {
-		args := []string{"-d 1ed5:"}
-		specifiedVendorInfo, err := getLspci(args)
-		if err != nil {
-			logger.Warning(err)
-			return false, err
-		}
-		if specifiedVendorInfo != "" {
-			vgaSupportGamma = true
+		// 部分显卡lspci没有VGA compatible controller相关数据，需要特殊处理
+		specialGPUVendorList := m.dsSpecialGPUVendorList
+		for _, vendor := range specialGPUVendorList {
+			args := []string{fmt.Sprintf("-d %s", vendor)}
+			specifiedVendorInfo, err := getLspci(args)
+			if err != nil {
+				logger.Warning(err)
+				vgaSupportGamma = false
+				continue
+			}
+			if specifiedVendorInfo != "" {
+				vgaSupportGamma = true
+				break
+			}
 		}
 	}
 	return vgaSupportGamma, nil
 }
+
+func (m *Manager) initDSettings(sysBus *dbus.Conn) {
+	ds := configManager.NewConfigManager(sysBus)
+
+	displayPath, err := ds.AcquireManager(0, DSettingsAppID, DSettingsDisplayName, "")
+	if err != nil {
+		logger.Warning(err)
+		return
+	}
+
+	dsDisplay, err := configManager.NewManager(sysBus, displayPath)
+	if err != nil {
+		logger.Warning(err)
+		return
+	}
+
+	getHDMIIsBuiltinConfig := func() {
+		v, err := dsDisplay.Value(0, DSettingsKeyHDMIIsBuiltinConfig)
+		if err != nil {
+			logger.Warning(err)
+			return
+		}
+		itemList := v.Value().([]dbus.Variant)
+		for _, i := range itemList {
+			m.dsHDMIIsBuiltinConfig = append(m.dsHDMIIsBuiltinConfig, i.Value().(string))
+		}
+	}
+	getBackLightMaxBrightnessChooseBigConfig := func() {
+		v, err := dsDisplay.Value(0, DSettingsKeyBackLightMaxBrightnessChooseBigConfig)
+		if err != nil {
+			logger.Warning(err)
+			return
+		}
+		itemList := v.Value().([]dbus.Variant)
+		for _, i := range itemList {
+			m.dsBackLightMaxBrightnessChooseBigConfig = append(m.dsBackLightMaxBrightnessChooseBigConfig, i.Value().(string))
+		}
+	}
+	getSpecialGPUVendorList := func() {
+		v, err := dsDisplay.Value(0, DSettingsKeySpecialGPUVendorListConfig)
+		if err != nil {
+			logger.Warning(err)
+			return
+		}
+		itemList := v.Value().([]dbus.Variant)
+		for _, i := range itemList {
+			m.dsSpecialGPUVendorList = append(m.dsSpecialGPUVendorList, i.Value().(string))
+		}
+	}
+	getLABCAutoStopEnable := func() {
+		v, err := dsDisplay.Value(0, DSettingsKeyLABCAutoStopEnabledConfig)
+		if err != nil {
+			logger.Warning(err)
+			return
+		}
+		m.dsLABCAutoStopEnable = v.Value().(bool)
+		logger.Info("labc auto stop enable:", m.dsLABCAutoStopEnable)
+	}
+	getHDMIIsBuiltinConfig()
+	getBackLightMaxBrightnessChooseBigConfig()
+	getSpecialGPUVendorList()
+	getLABCAutoStopEnable()
+
+	dsDisplay.InitSignalExt(m.sysSigLoop, true)
+	_, err = dsDisplay.ConnectValueChanged(func(key string) {
+		switch key {
+		case DSettingsKeyHDMIIsBuiltinConfig:
+			getHDMIIsBuiltinConfig()
+		case DSettingsKeyBackLightMaxBrightnessChooseBigConfig:
+			getBackLightMaxBrightnessChooseBigConfig()
+		case DSettingsKeySpecialGPUVendorListConfig:
+			getSpecialGPUVendorList()
+		case DSettingsKeyLABCAutoStopEnabledConfig:
+			getLABCAutoStopEnable()
+		}
+	})
+	if err != nil {
+		logger.Warning(err)
+	}
+}
+
+func (m *Manager) initHardwareInfo(sysBus *dbus.Conn) {
+	info := systeminfo.NewSystemInfo(sysBus)
+	dmi, err := info.DMIInfo().Get(0)
+	if err != nil {
+		logger.Warning(err)
+		return
+	}
+	m.dmiInfo = dmi
+	graphicList, err := graphic.GetGraphicList()
+	if err != nil {
+		logger.Warning(err)
+		return
+	}
+	for _, graphicInfo := range graphicList {
+		m.pciVendor = append(m.pciVendor, graphicInfo.Vendor)
+	}
+}
+
+func (m *Manager) GetMaxBacklightBrightness() int {
+	controllers := brightness.Controllers
+	if len(controllers) == 0 {
+		return 0
+	}
+	maxBrightness := controllers[0].MaxBrightness
+	if strv.Strv(m.dsBackLightMaxBrightnessChooseBigConfig).Contains(m.dmiInfo.ProductName) {
+		for _, controller := range controllers {
+			if maxBrightness < controller.MaxBrightness {
+				maxBrightness = controller.MaxBrightness
+			}
+		}
+	} else {
+		for _, controller := range controllers {
+			if maxBrightness > controller.MaxBrightness {
+				maxBrightness = controller.MaxBrightness
+			}
+		}
+	}
+	return maxBrightness
+}
