@@ -186,6 +186,13 @@ func (m *Manager) SetAndSaveBrightness(outputName string, value float64) *dbus.E
 	if !can {
 		return dbusutil.ToError(fmt.Errorf("the port %s cannot set brightness", outputName))
 	}
+
+	if m.dsLABCAutoStopEnable {
+		if enabled, _ := m.sysDisplay.AutoBacklightEnabled().Get(0); enabled {
+			m.sysDisplay.AutoBacklightEnabled().Set(0, false)
+		}
+	}
+
 	err := m.setBrightnessAndSync(outputName, value)
 	if err != nil {
 		logger.Warning(err)
@@ -214,6 +221,12 @@ func (m *Manager) SetBrightness(outputName string, value float64) *dbus.Error {
 		return dbusutil.ToError(fmt.Errorf("the port %s cannot set brightness", outputName))
 	}
 
+	if m.dsLABCAutoStopEnable {
+		if enabled, _ := m.sysDisplay.AutoBacklightEnabled().Get(0); enabled {
+			m.sysDisplay.AutoBacklightEnabled().Set(0, false)
+		}
+	}
+
 	err := m.setBrightnessAndSync(outputName, value)
 	if err != nil {
 		logger.Warning(err)
@@ -225,6 +238,9 @@ func (m *Manager) SetBrightness(outputName string, value float64) *dbus.Error {
 func (m *Manager) SetPrimary(outputName string) *dbus.Error {
 	logger.Debug("dbus call SetPrimary", outputName)
 	err := m.setPrimary(outputName)
+	if err == nil {
+		m.updateConnectMonitors()
+	}
 	return dbusutil.ToError(err)
 }
 
