@@ -18,7 +18,9 @@ import (
 	"time"
 
 	"github.com/davecgh/go-spew/spew"
+	dbus "github.com/godbus/dbus"
 	kwayland "github.com/linuxdeepin/go-dbus-factory/com.deepin.daemon.kwayland"
+	configManager "github.com/linuxdeepin/go-dbus-factory/org.desktopspec.ConfigManager"
 	"github.com/linuxdeepin/go-lib/dbusutil"
 	"github.com/linuxdeepin/go-lib/log"
 	"github.com/linuxdeepin/go-x11-client/ext/randr"
@@ -111,6 +113,14 @@ const (
 	OutputDeviceModePreferred = 1 << 1
 )
 
+const (
+	dsettingsID                             = "org.deepin.dde.wloutput.daemon"
+	dsettingsPath                           = "org.deepin.dde.wloutput.daemon"
+	dsettingsBrightnessAdjustTypeUnitsKey   = "brightnessAdjustType"
+
+	colorCurve                              = "colorcurve"
+)
+
 func (oi *KOutputInfo) getBestMode() ModeInfo {
 	var preferredMode *KModeInfo
 	for _, info := range oi.ModeInfos {
@@ -194,7 +204,7 @@ func (oi *KOutputInfo) getName() string {
 	return getOutputDeviceName(oi.Model, oi.Manufacturer)
 }
 
-func (oi *KOutputInfo) toMonitorInfo(mm *kMonitorManager) *MonitorInfo {
+func (oi *KOutputInfo) toMonitorInfo(mm *kMonitorManager, adjustType string) *MonitorInfo {
 	mi := &MonitorInfo{
 		Enabled:          oi.Enabled != 0,
 		ID:               oi.id,
@@ -230,7 +240,9 @@ func (oi *KOutputInfo) toMonitorInfo(mm *kMonitorManager) *MonitorInfo {
 
 	mi.UUID = oi.UUID
 	mi.UuidV0 = oi.UUID
-	if mi.Connected {
+
+	// 华为设备的亮度调节方式不是colorCurve,需要调用窗管接口，传递原始的UUID信息
+	if mi.Connected && adjustType == colorCurve {
 		stdName, err := mm.getStdMonitorName(mi.Name, mi.EDID)
 		if err != nil {
 			logger.Warningf("get monitor %v std name failed: %v", mi.Name, err)
@@ -376,6 +388,7 @@ func (m *Manager) removeMonitor(id uint32) *Monitor {
 type kMonitorManager struct {
 	hooks          monitorManagerHooks
 	sessionSigLoop *dbusutil.SignalLoop
+	systemSigLoop  *dbusutil.SignalLoop
 	mu             sync.Mutex
 	management     kwayland.OutputManagement
 	mig            *monitorIdGenerator
@@ -383,6 +396,7 @@ type kMonitorManager struct {
 	primary        uint32
 	// 键是 wayland 原始数据 model 字段处理过后的显示器名称，值是标准名。
 	stdNamesCache map[string]string
+	adjustBrightnessType string
 }
 
 func newKMonitorManager(sessionSigLoop *dbusutil.SignalLoop) *kMonitorManager {
@@ -390,6 +404,7 @@ func newKMonitorManager(sessionSigLoop *dbusutil.SignalLoop) *kMonitorManager {
 		sessionSigLoop: sessionSigLoop,
 		monitorMap:     make(map[uint32]*MonitorInfo),
 		stdNamesCache:  make(map[string]string),
+		adjustBrightnessType: "colorcurve",
 	}
 
 	sessionBus := sessionSigLoop.Conn()
@@ -403,6 +418,48 @@ func newKMonitorManager(sessionSigLoop *dbusutil.SignalLoop) *kMonitorManager {
 }
 
 func (mm *kMonitorManager) init() error {
+	// 主线wayland需要用colorcurve方式调节亮度，华为设备使用默认方案
+	sysBus, err := dbus.SystemBus()
+	if err != nil {
+		logger.Warning(err)
+	} else {
+		mm.systemSigLoop = dbusutil.NewSignalLoop(sysBus, 10)
+		mm.systemSigLoop.Start()
+
+		ds := configManager.NewConfigManager(sysBus)
+		wloutputPath, err := ds.AcquireManager(0, dsettingsID, dsettingsPath, "")
+		if err != nil {
+			logger.Warning(err)
+		} else {
+			wlouputManager, err := configManager.NewManager(sysBus, wloutputPath)
+			if err != nil {
+				logger.Warning(err)
+			} else {
+				getBrightnessAdjustType := func() {
+					v, err := wlouputManager.Value(0, dsettingsBrightnessAdjustTypeUnitsKey)
+					if err != nil {
+						logger.Warning(err)
+						return
+					}
+
+					mm.adjustBrightnessType = v.Value().(string)
+				}
+
+				getBrightnessAdjustType()
+
+				wlouputManager.InitSignalExt(mm.systemSigLoop, true)
+				_, err = wlouputManager.ConnectValueChanged(func(key string) {
+					if key == dsettingsBrightnessAdjustTypeUnitsKey {
+						getBrightnessAdjustType()
+					}
+				})
+				if err != nil {
+					logger.Warning(err)
+				}
+			}
+		}
+	}
+
 	mm.listenDBusSignals()
 	// init monitorMap
 	kOutputInfos, err := mm.listOutput()
@@ -410,7 +467,8 @@ func (mm *kMonitorManager) init() error {
 		logger.Warning(err)
 	}
 	mm.mu.Lock()
-	monitors := kOutputInfos.toMonitorInfos(mm)
+	monitors := kOutputInfos.toMonitorInfos(mm, mm.adjustBrightnessType)
+
 	for _, monitor := range monitors {
 		mm.monitorMap[monitor.ID] = monitor
 	}
@@ -423,33 +481,21 @@ func (mm *kMonitorManager) init() error {
 
 type KOutputInfos []*KOutputInfo
 
-func (infos KOutputInfos) toMonitorInfos(mm *kMonitorManager) []*MonitorInfo {
+func (infos KOutputInfos) toMonitorInfos(mm *kMonitorManager, adjustType string) []*MonitorInfo {
 	result := make([]*MonitorInfo, 0, len(infos))
 	for _, info := range infos {
-		result = append(result, info.toMonitorInfo(mm))
+		result = append(result, info.toMonitorInfo(mm, adjustType))
 	}
 	return result
 }
 
 func (mm *kMonitorManager) listOutput() (KOutputInfos, error) {
-	var outputJ string
-	var duration = 500 * time.Millisecond
-	// sometimes got the output list will return nil, this is the output service not inited yet.
-	// so try got 3 times.
-	for i := 0; i < 3; i++ {
-		data, err := mm.management.ListOutput(0)
-		if len(data) != 0 {
-			outputJ = data
-			break
-		}
-
-		if err != nil || len(data) == 0 {
-			logger.Warning("Failed to get output list:", err)
-		}
-		time.Sleep(duration)
-		duration += 100 * time.Millisecond
+	outputJ, err := mm.management.ListOutput(0)
+	if err != nil || len(outputJ) == 0 {
+		logger.Warning("Failed to get output list:", err)
 	}
-	logger.Debug("outputJ:", outputJ)
+
+	logger.Warning("outputJ:", outputJ)
 	outputInfos, err := unmarshalOutputInfos(outputJ)
 	if err != nil {
 		return nil, err
@@ -593,7 +639,7 @@ func (mm *kMonitorManager) handleOutputAdded(outputInfo *KOutputInfo) {
 	logger.Debugf("OutputAdded %#v", outputInfo)
 
 	mm.mu.Lock()
-	monitorInfo := outputInfo.toMonitorInfo(mm)
+	monitorInfo := outputInfo.toMonitorInfo(mm, mm.adjustBrightnessType)
 	mm.monitorMap[outputInfo.id] = monitorInfo
 	mm.mu.Unlock()
 
@@ -606,7 +652,7 @@ func (mm *kMonitorManager) handleOutputChanged(outputInfo *KOutputInfo) {
 	logger.Debugf("OutputChanged %#v", outputInfo)
 
 	mm.mu.Lock()
-	monitorInfo := outputInfo.toMonitorInfo(mm)
+	monitorInfo := outputInfo.toMonitorInfo(mm, mm.adjustBrightnessType)
 	mm.monitorMap[outputInfo.id] = monitorInfo
 	primary := mm.primary
 	mm.mu.Unlock()
