@@ -59,7 +59,7 @@ func Init(xConn *x.Conn, useWayland bool, inVM bool) {
 }
 
 func GetRecommendedScaleFactor() float64 {
-	if !_hasRandr1d2 {
+	if !_hasRandr1d2 || _useWayland {
 		return 1
 	}
 	resources, err := getScreenResources(_xConn)
@@ -379,7 +379,27 @@ func (mm *xMonitorManager) doDiff() {
 				}
 			}
 		} else {
-			logger.Warning("can not handle new monitor")
+			logger.Info("need add new monitor")
+			if mm.monitorChangedCbEnabled {
+				if mm.hooks != nil {
+					mm.mu.Unlock()
+					mm.hooks.handleMonitorChanged(monitor)
+					mm.mu.Lock()
+				}
+			} else {
+				logger.Debug("monitorChangedCb disabled")
+			}
+		}
+	}
+	newMap := toMonitorInfoMap(newMonitors)
+	for k, monitor := range oldMonitors {
+		_, ok := newMap[k]
+		if !ok {
+			// 需要移除的monitor
+			mm.mu.Unlock()
+			logger.Info("remove monitor:", monitor.ID)
+			mm.hooks.handleMonitorRemoved(monitor.ID)
+			mm.mu.Lock()
 		}
 	}
 }
@@ -573,15 +593,37 @@ func (mm *xMonitorManager) findFreeCrtc(output randr.Output, freeCrtcs map[randr
 	mm.mu.Lock()
 	defer mm.mu.Unlock()
 
+	// 优先选择最适合的crtc
 	for crtc, crtcInfo := range mm.crtcs {
 		isFree := freeCrtcs[crtc]
-		if isFree && outputSliceContains(crtcInfo.PossibleOutputs, output) {
+		if isFree &&
+			len(crtcInfo.PossibleOutputs) == 1 &&
+			crtcInfo.PossibleOutputs[0] == output {
 			freeCrtcs[crtc] = false
 			return crtc
 		} else if outputSliceContains(crtcInfo.Outputs, output) {
 			return crtc
 		}
 	}
+
+	min := math.MaxInt32
+	best := randr.Crtc(0)
+	for crtc, crtcInfo := range mm.crtcs {
+		isFree := freeCrtcs[crtc]
+		if isFree && outputSliceContains(crtcInfo.PossibleOutputs, output) {
+			if len(crtcInfo.PossibleOutputs) < min {
+				min = len(crtcInfo.PossibleOutputs)
+				best = crtc
+			}
+		} else if outputSliceContains(crtcInfo.Outputs, output) {
+			return crtc
+		}
+	}
+
+	if best != 0 {
+		freeCrtcs[best] = false
+		return best
+	}
 	return 0
 }
 
@@ -1171,6 +1213,7 @@ func (mm *xMonitorManager) handleOutputChanged(e *randr.OutputChangeNotifyEvent)
 		reply, err := mm.getOutputInfo(e.Output)
 		if err != nil {
 			logger.Warningf("get output %v info failed: %v", e.Output, err)
+			delete(mm.outputs, e.Output)
 			return
 		}
 		mm.outputs[e.Output] = (*OutputInfo)(reply)
