@@ -30,6 +30,7 @@ import (
 	"github.com/linuxdeepin/go-lib/log"
 	"github.com/linuxdeepin/go-lib/proxy"
 	x "github.com/linuxdeepin/go-x11-client"
+
 	"github.com/linuxdeepin/startdde/display"
 	"github.com/linuxdeepin/startdde/iowait"
 	"github.com/linuxdeepin/startdde/watchdog"
@@ -76,7 +77,27 @@ func reapZombies() {
 }
 
 func shouldUseDDEKWin() bool {
-	_, err := os.Stat("/usr/bin/kwin_no_scale")
+	var (
+		wmCmd string
+		err   error
+	)
+
+	// for unit test
+	if _gSettingsConfig == nil {
+		goto end
+	}
+	wmCmd = _gSettingsConfig.wmCmd
+	if len(wmCmd) != 0 {
+		_, err = os.Stat(strings.Split(wmCmd, " ")[0])
+		if err == nil {
+			return false
+		}
+		// fallback
+		_gSettingsConfig.wmCmd = ""
+	}
+
+end:
+	_, err = os.Stat("/usr/bin/kwin_no_scale")
 	return err == nil
 }
 
@@ -244,7 +265,7 @@ func greeterDisplayMain() {
 	if err != nil {
 		logger.Warning(err)
 	}
-	err = display.Start(service)
+	err = display.Start(nil, service)
 	if err != nil {
 		logger.Warning(err)
 	}
@@ -350,17 +371,10 @@ func main() {
 	}
 	logDebugAfter("before launchCoreComponents")
 
-	err = display.Start(service)
+	err = display.Start(xsManager, service)
 	if err != nil {
 		logger.Warning("start display part1 failed:", err)
 	}
-
-	// NOTE: always start pulseaudio
-	err = startPulseAudio()
-	if err != nil {
-		logger.Warning("failed to start pulseaudio:", err)
-	}
-
 	launchCoreComponents(sessionManager)
 
 	// 启动 display 模块的后一部分
@@ -372,6 +386,8 @@ func main() {
 	}()
 
 	go func() {
+		// NOTE: always start pulseaudio
+		startPulseAudio()
 		initSoundThemePlayer()
 		playLoginSound()
 	}()
