@@ -153,6 +153,25 @@ func stopRedshift() {
 	}
 }
 
+func sessionDBusStopUnit() {
+	// NOTE: Proactively stop the dbus service.
+	// Avoid long logout time : bug-188975
+	bus, err := dbus.SessionBus()
+	if err == nil {
+		systemdUser := bus.Object("org.freedesktop.systemd1", "/org/freedesktop/systemd1")
+		var jobPath dbus.ObjectPath
+		err = systemdUser.Call("org.freedesktop.systemd1.Manager.StopUnit",
+			dbus.FlagNoAutoStart, "dbus.service", "replace-irreversibly").Store(&jobPath)
+		if err != nil {
+			logger.Warning("failed to stop dbus.service:", err)
+		} else {
+			logger.Info("sessionDBusStopUnit jobPath : ", jobPath)
+		}
+	} else {
+		logger.Warning(err)
+	}
+}
+
 func (m *SessionManager) prepareLogout(force bool) {
 	if !force {
 		err := autostop.LaunchAutostopScripts(logger)
@@ -160,12 +179,10 @@ func (m *SessionManager) prepareLogout(force bool) {
 			logger.Warning("failed to run auto script:", err)
 		}
 	}
-
+	watchdog.Stop()
 	killSogouImeWatchdog()
 	// kill process LangSelector ,because LangSelector will not be kill by common logout
 	killLangSelector()
-	// quit at-spi-dbus-bus.service
-	quitAtSpiService()
 	stopBAMFDaemon()
 	stopRedshift()
 	quitObexService()
@@ -178,6 +195,9 @@ func (m *SessionManager) prepareLogout(force bool) {
 	} else {
 		quitPulseAudio()
 	}
+	// quit at-spi-dbus-bus.service
+	quitAtSpiService()
+	sessionDBusStopUnit()
 }
 
 // kill process LangSelector by cmd "pkill -ef -u $UID /usr/lib/deepin-daemon/langselector"
@@ -375,7 +395,9 @@ func (m *SessionManager) RequestSuspend() *dbus.Error {
 	if err == nil {
 		// no suspend
 		time.Sleep(time.Second)
-		setDPMSMode(false)
+		if _gSettingsConfig.needQuickBlackScreen {
+			setDPMSMode(false)
+		}
 		return nil
 	}
 
@@ -1229,6 +1251,7 @@ func (m *SessionManager) killLockFront() error {
 }
 
 func setDPMSMode(on bool) {
+	logger.Info("[setDPMSMode] on : ", on)
 	var err error
 	if _useWayland {
 		if !on {
@@ -1384,8 +1407,8 @@ func initXEventMonitor() {
 	sigLoop.Start()
 	xEvent.InitSignalExt(sigLoop, true)
 	xEvent.ConnectButtonPress(func(button int32, x int32, y int32, id string) {
-		// 5表示鼠标中间
-		if button == 5 {
+		// 2表示按下鼠标中键，4表示鼠标中键上滑，5表示鼠标中键下滑
+		if button == 2 || button == 4 || button == 5 {
 			setDPMSMode(true)
 		}
 	})
