@@ -67,6 +67,10 @@ func Start(getLockedFn func() bool, useKwin bool) {
 		logger.Warning(err)
 	}
 	time.AfterFunc(10*time.Second, func() {
+		if _manager == nil {
+			logger.Warning("_manager is nil.")
+			return
+		}
 		for _, task := range _manager.dbusTasks {
 			err = task.Launch()
 			if err != nil {
